using System;
using UnityEngine;
using UnityEngine.Experimental.Rendering;
using UnityEngine.Events;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Linq;
using MVR;
using MeshVR;
using MVR.FileManagementSecure;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using GPUTools.Cloth.Scripts;
using Unity.Collections;
using AssetBundles;
using System.Diagnostics;

/// <summary>

/// </summary>

namespace Stopper {
public class FemaleChestPhysicsForMale : MVRScript
{
	Atom femalePrefabAtom;
	DAZCharacterRun characterRun;
	DAZPhysicsMesh breastDazMesh;
	
	
	GameObject maleMorphers = null;
	GameObject femaleMorphers = null;
	
	GameObject breastPhysicsMesh = null;
	
	public override void Init() {
		StartCoroutine("TransPlant");
	}
	
	private IEnumerator TransPlant(){

		foreach(GameObject go in GetAllChildren(containingAtom.gameObject)){
			if(go.name == "BreastPhysicsMesh"){
				breastPhysicsMesh = go;
			}
			
			if(go.name == "MaleMorphers"){
				maleMorphers = go;
			}
			
			if(go.name == "FemaleMorphers"){
				femaleMorphers = go;
			}
		}
		
		AutoColliderGroup maleChestAutoColliderGroup = null;
		foreach(var autoColliderGroup in containingAtom.GetComponentsInChildren<AutoColliderGroup>()){
			if(autoColliderGroup.name.Contains("MaleAutoColliderschest")){
				maleChestAutoColliderGroup = autoColliderGroup;
			}
		}
		
		
		var skin = containingAtom.GetComponentInChildren<DAZMergedSkinV2>();
		
		
		skin.useMultithreadedSkinning = true;
		characterRun = containingAtom.GetComponentInChildren<DAZCharacterRun>();
		var breastDazMesh = breastPhysicsMesh.GetComponent<DAZPhysicsMesh>();
		
		// Take female breast physics and put them under MaleMorphers
		breastPhysicsMesh.transform.parent = maleMorphers.transform;
		breastDazMesh.skin = skin;
		
		// Ignore male chest colliders
		foreach(var vertGroup in breastDazMesh.softVerticesGroups){
			var ignoreAutoColliderGroups = vertGroup.ignoreAutoColliderGroups.ToList();
			if(!ignoreAutoColliderGroups.Contains(maleChestAutoColliderGroup)){
				ignoreAutoColliderGroups.Add(maleChestAutoColliderGroup);
			}
			vertGroup.ignoreAutoColliderGroups = ignoreAutoColliderGroups.ToArray();
		}
		
		breastDazMesh.InitColliders();
		
		// Register the mesh to update with CharacterRun
		if(characterRun != null){
			var newMeshes = characterRun.physicsMeshes.ToList();
			if(!newMeshes.Contains(breastDazMesh)){
				newMeshes.Add(breastDazMesh);
			}
			characterRun.physicsMeshes = newMeshes.ToArray();
		}
		
		breastDazMesh.Init();
		breastDazMesh.enabled = true;
		breastDazMesh.on = true;
		
		
		var characterSelector = containingAtom.GetComponentInChildren<DAZCharacterSelector>();
		var morphBank = characterSelector.morphBank1;
		var bodyScaleMorph = morphBank.GetMorphByUid("Body Scale");
		
		yield return new WaitForSeconds(1f);
		
		if(bodyScaleMorph != null){
			bodyScaleMorph.morphValue += 0.1f;
			yield return new WaitForFixedUpdate();
			yield return new WaitForFixedUpdate();
			yield return null;
			yield return null;
			bodyScaleMorph.morphValue -= 0.1f;
		}
		
		yield break;
	}
	
	private void OnDestroy(){
		if( femaleMorphers != null){
			if(breastPhysicsMesh != null){
				breastPhysicsMesh.transform.parent = femaleMorphers.transform;
			}
		}
		
		if(characterRun != null && breastDazMesh != null){
			var newMeshes = characterRun.physicsMeshes.ToList();
			newMeshes.Remove(breastDazMesh);
			characterRun.physicsMeshes = newMeshes.ToArray();
		}
	}
	
	List<GameObject> GetAllChildren(GameObject parent, List<GameObject> objectList = null)
	{
		if (objectList == null) objectList = new List<GameObject>();
	  
		foreach (Transform child in parent.transform) {
			objectList.Add(child.gameObject);
			GetAllChildren(child.gameObject, objectList);
		}
		return objectList;
	}
}
}